C Calculation of the paraequilibrium Ae3 temperature
C H. K. D. H. Bhadeshia, University of Cambridge
C
C AC is in weight percent, C, Si, Mn, Ni, Mo, Cr, V
C AC(8) can be Fe by difference
C AE3T is the paraequilibrium Ae3' temperature
C AINCET is the intercept in linear regression
C DTQ and DXEQ store temperature (centigrade)/XEQ (mol. frac.) pairs
C F is the free energy difference (ferrite - austenite) of the same
C    composition when carbon is removed.
C When I is zero, the calculations are repeated for higher accuracy
C J8 is the number of XEQ/T pairs computed for the specified T range
C R is the universal gas constant, J/K/mol
C SLOPE is the slope in the linear regression
C T is the absolute temperature
C W is the carbon-carbon interaction energy in austenite
C XEQ is the paraequilibrium carbon concentration of austenite
C
      SUBROUTINE MAP_STEEL_AA3(AC,AE3T,R,I1,I2,I3)
C
      IMPLICIT NONE
      DOUBLE PRECISION DTQ(40),DXEQ(40),XEQ,C(8),AC(8)
      DOUBLE PRECISION R,W,XBAR,T10,T20,T,F,MAP_STEEL_ENERGY
      DOUBLE PRECISION TEQ,MAP_STEEL_AFEG,MAP_STEEL_DAFEG,ETEQ
      DOUBLE PRECISION CTEMP,AINCET,SLOPE,CORR,AJ,AE3T
      INTEGER I1,I2,I3,J8,I,J,IT
C
C This ensures that AC remains unchanged after being called by OMEGA
         DO 10 J=1,8
         C(J)=AC(J)
 10      CONTINUE
C
      I=0
      CALL MAP_STEEL_OMEGA(C,W,XBAR,T10,T20,0)
C
C It is assumed that the Ae3' temperature lies between I1 and I2, in Kelvin
C I3 defines the temperature interval over which the calculations are conducted
C
C Take a guess for XEQ
 20   XEQ=0.1
      J8=0
C
C Calculate a set of XEQ values as a function of T
C For method, see equation 2, Acta Metallurgica, 28 (1980) 1265-1273.
C   The equation is solved by iteration. Hence, AFEG is the natural logarithm
C   of the activity of iron in austenite, DAFEG is the differential of that
C   with respect to carbon (XEQ). The iteration is based on the Newton-Raphson
C   method. The iteration stops when XEQ becomes very small.
C   The precise Ae3' temperature is found by linear regression of the
C   temperature XEQ pairs by the ANAL.
C
        DO 30 IT=I1,I2,I3
          J8=J8+1
          T=IT
          F=MAP_STEEL_ENERGY(T,T10,T20)
          AJ=1.0-DEXP(-W/(R*T))
 50         TEQ=MAP_STEEL_AFEG(XEQ,AJ)*R*T-F
            IF (DABS(TEQ) .LT. 1.0) GOTO 60
            ETEQ=MAP_STEEL_DAFEG(XEQ,AJ)*R*T
            XEQ=XEQ-TEQ/ETEQ
            GOTO 50
C Calculations stop when XEQ becomes very small
 60       IF (XEQ .LT. 0.001) GOTO 70
          CTEMP=IT-273.0
          DTQ(J8)=CTEMP
          DXEQ(J8)=XEQ
 30     CONTINUE
 70     J8=J8-1
C
C Linear regression to find exact Ae3' temperature for alloy of interest
        CALL MAP_UTIL_ANALY(J8,0,AINCET,SLOPE,CORR,DTQ,DXEQ)
        AE3T=(C(1)-AINCET)/SLOPE
C
C Repeat analysis to a finer precision for Ae3'
        IF(I .GT. 0) GOTO 100
        I=AE3T
        I1=I-9+273
        I2=I+9+273
        I3=3
      GOTO 20
100   RETURN
      END
