C Function MAP_STEEL_FTO1
C
C Produced by :-
C
C H. K. D. H. Bhadeshia
C
C Department of Materials Science and Metallurgy,
C Pembroke Street, Cambridge CB2 3QZ, U. K.
C 31 March 1987
C
C Calculates the free energy change accompanying the transformation
C from austenite to ferrite of the same chemical composition, including
C a Zener ordering term. The latter describes the ordering of carbon
C atoms, that leads the body-centered cubic lattice of ferrite becoming
C body-centered tetragonal. The degree of ordering increases as the
C carbon concentration increases, or as the transformation temperature
C decreases.
C
C References:
C
C  1. C. Zener Trans. AIME 167 (1946) 550.
C
C  2. J. C. Fisher, Metals Transactions 185 (1949) 688-690.
C
C  3. H. K. D. H. Bhadeshia and D. V. Edmonds
C      Acta Metallurgica 28 (1980) 1265-1273.
C      Also see laboratory book 7.
C
C----------------------------------------------------------------------
C
       DOUBLE PRECISION FUNCTION MAP_STEEL_FTO1(H,S,X,T,W,W1,
     & H1,S1,F,AJ,AJ1,R)
C
       IMPLICIT NONE
C
C Input arguments.
C
       DOUBLE PRECISION AJ, AJ1, F, H, H1, R, S, S1, T, W, W1, X
C
C Local variables.
C
       DOUBLE PRECISION D, D11, T60, ZENER, ZEN1, ZEN2
C
C Begin function :-
C
       D  = SQRT(1-2*(1+2*AJ)*X+(1+8*AJ)*X*X)
       D11= SQRT(9-6*X*(2*AJ1+3)+(9+16*AJ1)*X*X)
C
C Calculate T60, which is the value of the actual temperature T
C   divided by the critical temperature beyond which there is no
C   Zener ordering. X is the mole fraction of carbon in the ferrite.
C If T60>1, the transformation occurs above the ordering temperature
C   and Zener ordering does not occur.
C
       T60 = T*(1-X)/(28080*X)
C
       IF (T60 .GT. 1.0) THEN
          GOTO 30
       END  IF
C
C Calculate the contribution to the free energy change due to
C  Zener ordering.
C
       IF (T60 .GT. 0.25) THEN
          GOTO 10
       END  IF
C
C Polynomial fit to data in Table 2 of Fischer (1949)
C
       ZEN1=0.2307+42.7974*T60-233.8631*(T60**2)+645.4485*(T60**3)
     & -954.3995*(T60**4)+711.8095*(T60**5)-211.5136*(T60**6)
       ZEN2=-2.6702+45.6337*T60-225.3965*(T60**2)+567.7112*(T60**3)
     & -771.6466*(T60**4)+538.1778*(T60**5)-151.3818*(T60**6)
       GOTO 20
C
10     CONTINUE
       ZEN2 = 1
       ZEN1 = 3.295
C
20     CONTINUE
       ZENER=(((ZEN2*X)**2)*(-50898.56)/(1-X))+ZEN1*T*X*0.6623741
C
C Convert from calories per mole to Joules per mole
C
       ZENER=ZENER*4.187
       GOTO 40
C
30     CONTINUE
       ZENER=0.0
C
C Calculation of the Zener ordering term complete.  Calculate the free
C  energy change accompanying the change from austenite to ferrite of the
C  same composition, including Zener ordering term. See Bhadeshia
C  and Edmonds [3].
C
40     CONTINUE
C
       MAP_STEEL_FTO1 = X*R*T*DLOG(X*X)+X*(H1-(H)-(S1-(S)
     & )*T+4*W1-6*W)-R*T*(1-X)*DLOG((1-X)**4)+5*R*T*(1-2*X)*DLOG(1-2*X)
     & -R*T*X*DLOG(((D-1+3*X)/(D+1-3*X))**6)-R*T*(1-X)*DLOG(((1-2*AJ+(4
     & *AJ-1)*X-D)/(2*AJ*(2*X-1)))**6)+3*R*T*X*DLOG(3-4*X)+R*T*
     & X*DLOG(((D11-3+5*X)/(D11+3-5*X))**4)+(1-X)*F+ZENER
C
C
       RETURN
       END
