C
C
C     Program to prepare data to be used during the
C     calculations of phase transformations during
C     cooling from austenite
C
C	VERSION 2.0 *multiple choice of cooling data input*
C
C	D.Gaude-Fugarolas
C	Cambridge February 2002
C
C
C
C
      PROGRAM PREPARE
C
C
      IMPLICIT NONE
C
C     ....Variables used........
C
C
      INTEGER IN1, IN2, IN3, OUT1, OUT2, I, 
     $     I1, I2, I3, J, JTL, JTH, JT
C
      INTEGER OPTION
C
      DOUBLE PRECISION RATEFIX
C
      DOUBLE PRECISION  AE3T, W, W1, XBAR, T20, T10, BS, MS, XTOSL, 
     $      XTOCN, XTOSL2, XTOCN2, NEWTIME, NEWTEMP, OLDTIME, 
     $      OLDTEMP, RATEUP, RATEDO
C
      DOUBLE PRECISION COMP(8),TEMPCOMP(8),TARGETT(3),SUBRATE(3)
C
      DOUBLE PRECISION EQGRSIZE, R
C
C
C     ....Parameters and initialised variables......
C
      PARAMETER (IN1=31)
      PARAMETER (IN2=32)
      PARAMETER (IN3=33)
      PARAMETER (OUT1=34)
      PARAMETER (OUT2=35)
C
C
C     ...inclusion of other subroutines ...
C
C
C     ...Gas constant...
C
      R=8.31451D+00
C
C     ...Equiaxed austenite grain size...
C     ...if different modiffy to be asked for...
C
      EQGRSIZE =100.0D-6
C
C     ...others...
C
      I1=573
      I2=1173
      I3=30
      J=0
      JTL=273
      JTH=873
      JT=40
C
      W1=48570.0D+00
C
C
      OPTION = 5
C
C
C     ....PROGRAM.......
C
C     ...Open files....
C
C
      OPEN(UNIT=IN1, FILE="compos.dat")
      OPEN(UNIT=IN2, FILE="temper.dat")
      OPEN(UNIT=IN3, FILE="grainsz.dat")
      OPEN(UNIT=OUT1, FILE="hermes.dat")
      OPEN(UNIT=OUT2, FILE="inpstruct.dat")
C
C
C     ....read equiaxed austenite grain size from 'grainsz.dat'....
C
      READ(IN3,*) EQGRSIZE
C
      CLOSE(IN3)
C
C     ....read composition from 'compos.dat'...
C
C
      WRITE(*,*)"Reading composition from compos.dat"
C
      COMP(8)=100.0D+00
C
      DO 10,I=1, 7
C
         READ(IN1,*) COMP(I)
C
         COMP(8) = COMP(8) - COMP(I)
C
 10   CONTINUE
C
      CLOSE (IN1)
C
C
C     calculate paraequilibrium Ac3 (MAP_STEEL_AA3)
C
C
      WRITE(*,*)"Calculation of Ac3"
C
      DO 121 I=1, 8
C
         TEMPCOMP(I)=COMP(I)
C
 121  CONTINUE
C
      CALL MAP_STEEL_AA3(TEMPCOMP, AE3T, R, I1, I2, I3)
C
C
C     ....calculation of Bs and Ms, using MAP_STEEL_OMEGA
C           and MAP_STEEL_BSMS...
C
C
      WRITE(*,*)"Calculation of Bs and Ms"
      WRITE(*,*)" (It might take a while)"
C
      DO 120 I=1, 8
C
         TEMPCOMP(I)=COMP(I)
C
 120  CONTINUE
C
C
      CALL MAP_STEEL_OMEGA(TEMPCOMP, W, XBAR, T10, T20, J)
C
C
      CALL MAP_STEEL_BSMS(T10, T20, W, W1, JTL, JTH, JT,
     $      BS, MS, XBAR, XTOSL, XTOCN, XTOSL2, XTOCN2, R)
C
C
C
C     ...3 options to input thermal history...
C
C
      WRITE(*,*)
      WRITE(*,*)
      WRITE(*,*)
C
      WRITE(*,*)"Phase transformation simulator on cooling proceses"
C
 24   WRITE(*,*)"Please, choose 1, 2, 3 or 0"
      WRITE(*,*)"(1)-Intro constant cooling rate"
      WRITE(*,*)"(2)-Smooth cooling history file"
      WRITE(*,*)"(3)-Noisy cooling history file"
      WRITE(*,*)"(0)-End program"
C
      WRITE(*,*)
      WRITE(*,*)
C
      READ(*,*) OPTION
C
C
      IF (OPTION .EQ. 1) THEN 
             GOTO 21
         ELSE IF (OPTION.EQ.2) THEN
            GOTO 22
         ELSE IF (OPTION.EQ.3) THEN
            GOTO 23
         ELSE IF (OPTION.EQ.0) THEN
            GOTO 99
         ELSE 
            GOTO 24
      ENDIF
C
C
      GOTO 25
C
C    ...Option (1) manual input of cooling rate...
C
C
 21   WRITE(*,*)"Please, intro cooling rate in degrees C per second"
      READ(*,*) RATEFIX
C
      RATEUP = RATEFIX
      RATEDO = RATEFIX
C
      CLOSE(IN2)
C
      GOTO 25
C
C     ...Option (2) determination of cooling rate from 'smooth' data
C
C
C     ...Start reading (time, temperature) pairs, to determine
C        the equivalent cooling rates. Each equivalent cooling 
C        rate is the average od the cooling rates at each end
C        of the intervals Ac3-Bs and Bs-Ms. It is assumed that
C        there's only one column of temperatures, that they 
C        decrease continuously and that the first temperature
C        value is higher than Ac3 (otherwise, the first rate
C        is calculated using the two first pairs (t, T).     ...
C
C
 22   WRITE(*,*)"Reading thermal history and determining" 
      WRITE(*,*)"cooling rates"
C
      I=1
C
      TARGETT(1)=AE3T
      TARGETT(2)=BS
      TARGETT(3)=MS
C
C     ...skip header's line...
C
      READ(IN2,*)
C
C     ...starts reading (t,T) pairs ....
C
      READ(IN2, *) OLDTIME, OLDTEMP
C
 122  IF(I.LE.3) THEN
C
          READ(IN2,*) NEWTIME, NEWTEMP
C
          IF(NEWTEMP.LT.TARGETT(I)) THEN
C
             SUBRATE(I) = ((OLDTEMP - NEWTEMP)/(NEWTIME - 
     $             OLDTIME))
C
C
             I = I + 1
C
          ENDIF
C
          OLDTIME = NEWTIME
          OLDTEMP = NEWTEMP
C
          GOTO 122
C
      ENDIF
C
      RATEUP = 0.5*(SUBRATE(1)+SUBRATE(2))
      RATEDO = 0.5*(SUBRATE(2)+SUBRATE(3))
C
        write(*,*)"rrrrrrrrrrrrrrrrrrrrr", SUBRATE(1),SUBRATE(2), ratedo


      CLOSE(IN2)
C
      GOTO 25
C
C
C
C**
C     ...Option (3) determination of cooling rate from 'noisy' data
C
C
C     ...Start reading (time, temperature) pairs, to determine
C        the equivalent cooling rates. Each equivalent cooling 
C        rate determined from the temperature and time values at each
C        end of the intervals Ac3-Bs and Bs-Ms. It is assumed that
C        there's only one column of temperatures, that they 
C        decrease continuously and that the first temperature
C        value is higher than Ac3 (otherwise, the first rate
C        is calculated using the two first pairs (t, T).     ...
C
C
  23  WRITE(*,*)"Reading thermal history and determining" 
      WRITE(*,*)"cooling rates"
C
        I=1
C
      TARGETT(1)=AE3T
      TARGETT(2)=BS
      TARGETT(3)=MS
C
C     ...skip header's line...
C
      READ(IN2,*)
C
C     ...starts reading (t,T) pairs ....
C
  26  READ(IN2, *) NEWTIME, NEWTEMP
C
C
      IF(NEWTEMP.LT.TARGETT(I)) THEN
C
         IF(I.GT.1) THEN
C
            SUBRATE(I-1) = ((OLDTEMP-NEWTEMP)/(NEWTIME-OLDTIME))
C
         ENDIF
C
         OLDTIME = NEWTIME
         OLDTEMP = NEWTEMP
C
         I = I+1
C
C  
      ENDIF
C
      IF(I.GT.3) GOTO 12
C
      GOTO 26
C
C
  12   RATEUP = SUBRATE(1)
       RATEDO = SUBRATE(2)
C
      CLOSE(IN2)
C
C**
C
C
C       ...after all the calculations, it's time to 
C          store the results into a file (hermes.dat)
C          and to prepare the input file for MAP_STEEL_
C          STRUCTURE, called 'inpstruct.dat'.      .....
C
C
   25   WRITE(*,*)"Writing files"
C     
        DO 30 I=1, 7
C
           WRITE(OUT2,*) COMP(I)
C
 30     CONTINUE
C
        WRITE(OUT2,*) 0.1
        WRITE(OUT2,*) 500
C
C   Next value is the interphase energy between austenite and
C   ferrite (and cementite), and can be used to adjust slightly 
C   the final fit of the model
C
        WRITE(OUT2,*) 0.033
C
        WRITE(OUT2,*) 200.0
        WRITE(OUT2,*) 0.333
        WRITE(OUT2,*) EQGRSIZE
        WRITE(OUT2,*) 1.0D-8
        WRITE(OUT2,*) 0.45
        WRITE(OUT2,*) 01D-6
        WRITE(OUT2,*) 0.0
        WRITE(OUT2,*) 0.0
        WRITE(OUT2,*) 0
        WRITE(OUT2,*) 2
        WRITE(OUT2,*) RATEUP*60
C
        CLOSE(OUT2)
C
C
C     ....writting 'hermes.dat' ....
C
C
C     
        DO 40 I=1, 7
C
           WRITE(OUT1,*) COMP(I)
C
 40     CONTINUE
C
C       ...  space...
C
        WRITE(OUT1,*)
C 
        WRITE(OUT1,*) AE3T
        WRITE(OUT1,*) BS
        WRITE(OUT1,*) MS
C
        WRITE(OUT1,*)
C
        WRITE(OUT1,*) RATEUP
        WRITE(OUT1,*) RATEDO
C
        CLOSE(OUT1)
C
C
C     ...end of program ...
C
C
  99    STOP
C
        END
C
C******************************************************************
C******************************************************************
C******************************************************************
C
C
C     ...Include statements...
C
C
C
      INCLUDE 'aa3-b.f'
      INCLUDE 'afeg-b.f'
      INCLUDE 'analy-b.f'
      INCLUDE 'axto-b.f'
      INCLUDE 'bsms-b.f'
      INCLUDE 'dafeg-b.f'
      INCLUDE 'energy-b.f'
      INCLUDE 'fto1-b.f'
      INCLUDE 'g91-b.f'
      INCLUDE 'omega-b.f'
C
