Copyright: Dr. H. K. D. H. Bhadeshia, University of Cambridge, 1994
C
C	Modified (*) by D.Gaude-Fugarolas to build an integrated  
C	model of transformations during cooling from austenite.
C			Cambridge November 2001
C
C	Uses Koistiner & Marburger method to determine Ms and
C        the amount of martensite and retained austenite
C
C
C Continuously annealed steels
C
C Allows initial isothermal transformation (set TISO & TIMISO)
C before continuous cooling transformation to a mixed microstructure
C of bainite, martensite and retained austenite, toghether with 
C an amount of allortiomorphic ferrite which is an input
C
C AVER is the program version number
C CBAR is the average carbon concentration in the steel, wt.%
C DT represents the Schiel sum. Transformation is trigerred during
C  continuous cooling when this is equal to unity
C
C IA is the number of TTT curves calculated. IA increases as Carbon
C IB represents the temperature, small when T low
C JT sets the temperature interval (reduce to improve accuracy)
C JTHIGH is a temperature above Bs
C JTLOW is a temperature at which TTT calculation stopped
C  to follow transformation behaviour, set a series of values of JTLOW
C TCAMB is the ambient temperature for retained austenite calculation
C TISO is the isothermal transformation temperature, C
C TIMISO is the isothermal heat treatment time, s
C VBISO volume fraction of bainitic ferrite produced isothermally
C VPRIOR volume fraction of ferrite present before bainite is produced
C
C
C
      IMPLICIT REAL*8 (A-H,K-Z), INTEGER (I,J)
      DOUBLE PRECISION C(8),AC(8),KTEMP(200),SHEART(200),
     &  CTEMP(100,200),SHEATT(100,200),DT(100),
     &  VOLB22(100),MST(100),BST(100),CC(8),DTISO(1000)
      INTEGER JY(50)
C

C
      OPEN(UNIT=4, FILE='A')
      OPEN(UNIT=2, FILE='B')
C*
      OPEN(UNIT=12, FILE='evol2.tmp')
C*
C
16         AVER=2.9
C
C JCALC represents the number of 'C' curves computed for the 
C  bainite reaction, as a function of the extent of reaction
C  It has a maximum value of 100, set by matrix dimensions.
C  Each 'C' curve represents transformation from austenite whose
C  carbon concentration is enriched by 0.05 wt.%
C 

              JCALC=50
C
              JT=20
              JTHIGH=873
              JTLOW=373
              JP=0
              TCAMB=20.0D+00
C
C Initialise DTISO
            DO 19 I=1,JCALC
19          DTISO(I)=0.0D+00
C initialisation complete
C
C JNOISO is to be set to 0 if isothermal transformation to bainite precedes 
C continous cooling from the austenitisation temperature
           JNOISO=1
C
C JNUM divides the temperature interval into a number of equal steps
           JNUM=(JTHIGH-JTLOW)/JT
  

C
      WRITE(*,20)
20    FORMAT(10X,' Input isothermal transformation temperature, C'/
     &       17X,' prior volume fraction of allotriomorphic ferrite'/
     &       17X,' isothermal transformation time, s'/
     &       17X,' JNOISO  (= 0 if isothermal transformation to',
     &           ' bainite precedes continuous cooling)   '/)
      READ(*,*,ERR=17)TISO,VPRIOR,TIMISO,JNOISO
      read(*,*) cool

      CALL RCOMP(C,ID,0)


C
C Calculate carbon locked inside ferrite formed at 700 Centigrade, wt.%
C XALPH calculates in mole fraction, so converted to wt.% by multiplying
C   by the ratio of atomic weights of carbon and iron, and by 100
C
      CALPHA=(1201.15/55.84)*XALPH(973.0D+00)
C
C* ...Allow for carbon-enrichment of austenite due to prior ferrite
C*    when the Si content is high enough to prevent the
C*    formation of carbides. For steels with a silicon content 
C*    lower than 1.5 wt.% the excess of carbon is released by 
C*    carbide formation and austenite is not considered to get 
C*    enriched by carbon. (see Bhadeshia and Edmonds 1979[ao], B1, 
C*    Seal & Honeycombe-1958, others).
C*
C*
      IF(C(2).GE.1.5) THEN
C*
         CRICH = (C(1)-CALPHA*VPRIOR)/(1.0D+00 - VPRIOR)
         CBAR=C(1)
         C(1)=CRICH
C*
      ELSE 
C*
         CRICH = C(1)
         CBAR = C(1)
C*
      ENDIF
C*
C*
C 
           DO 12 IC=1,8
12         CC(IC)=C(IC)
C
C -----------------------------------------------------
C Begin calculating the C-curves for bainite
C
      DO 2 J = 1, JCALC
C
            IF(J .GT. 1) THEN
C             Increment the austenite carbon 
C             concentration by 0.05 wt.% for
C             each successive 'C' curve 
              C(1)=C(1)+0.05D+00
            ENDIF
C
C Calculate the 'C' curve for bainite, using carbon 
C concentration given by the current value of C(1).
C This also gives values of T10, T20, W and W1 which
C are required for later calculations.
C
         CALL BAINTT(KTEMP,SHEART,C,AC,JT,JTHIGH,
     &                         JTLOW,T10,T20,W,W1)
C
C Calculate the Bs (bainite-start) and Ms (Martensite-start)
C  temperatures for the austenite, in degrees centigrade.
C Note that the slopes XTOSL and XTOSL2, and the constants
C XTOCN and XTOCN2 are for the T-zero and T-zero' curves
C respectively of the Temperature versus Carbon plot on
C the paraequilibrium phase diagram. 
C
      IF(J .EQ. 1)XBAR=AC(1)
      CALL BSMS(T10,T20,W,W1,JTLOW,JTHIGH,JT,BS,MS,
     &          AC(1),XTOSL,XTOCN,XTOSL2,XTOCN2,JP,J)
C
         MST(J)=MS
         BST(J)=BS
C
C Minimum Ms allowed is 30 centigrade
      IF(MS .LT. 30.0D+00)GOTO 15
C
C Calculate the limiting fractions of bainitic ferrite,
C   given the T-zero and T-zero' curves. Apply the lever
C   rule to these curves, the carbon concentration of the
C   bainitic ferrite given by the function XALPH.
C BXTO and CXTO are the T-zero and T-zero' carbon 
C   concentrations respectively.
C
      BXTO=BS*XTOSL+XTOCN
      CXTO=BS*XTOSL2+XTOCN2
      VOLB=(BXTO-XBAR)/(BXTO-XALPH(BS+273.0))
      VOLB2=(CXTO-XBAR)/(CXTO-XALPH(BS+273.0))
      VOLB22(J)=VOLB2
C
      WRITE(*,18)BS,MS,VOLB,VOLB2,BXTO,CXTO
18    FORMAT(' Bs = ',F5.0,'  Ms = ',F5.0,
     &       ' VOLXTO = ',F5.2,' VOLXTOPRIME = ',F5.2,
     &       ' XTO = ',F7.4,'  XTOPRIME = ',F7.4//
     &  '    CTEMP  SHEATT (between Ms & Bs of Residual Austenite)')
C
C
C Start by making the incubation period of C-curve "infinite"
            DO 1 I=1,JNUM
                SHEATT(J,I)=1.0D+22
C
C
              IF(KTEMP(I) .GT. BS +273.0 .OR. 
     &          KTEMP(I) .LT. MS+273.0) THEN
                CTEMP(J,I)=KTEMP(I)-273.0D+00
              ELSE
                CTEMP(J,I)=KTEMP(I)-273.0D+00
                SHEATT(J,I)=SHEART(I)
                WRITE(*,3)CTEMP(J,I),SHEATT(J,I)
              ENDIF
C
1           CONTINUE
C
2     CONTINUE
C
C Calculation of C-Curves for bainite complete
C --------------------------------------------
C 
C
C Isothermal transformation to bainite before continuous cooling
C No isothermal bainite if isothermal transformation temperature
C   above the bainite-start temperature
15         IF(TISO .GT. BST(1))THEN
             JNOISO=1
             WRITE(*,21)TISO,BST(1)
             GOTO 8
           ENDIF
C
C Integer IE is needed to assign temperature to the array SHEATT
C 
      IE=DINT((TISO-JTLOW+273.0)/JT)+1
C
C Calculate the Schiel parameter DTISO for isothermal transformation
C  for a period TIMISO for each of the 'C' curves
C

           DO 14 IA=1,JCALC
               IF(SHEATT(IA,IE) .GT. 0.0D+00)THEN
                 DTISO(IA)=TIMISO/SHEATT(IA,IE)
               ENDIF
14         CONTINUE
C
C End of isothermal transformation to bainite
C
C
8     DO 4 IA=1,JCALC
C        Include isothermal transformation
         DT(IA)=DTISO(IA)
C        Initialize JY to zero to represent Scheil parameter < 1
         JY(IA)=0
C
C
C      Work from highest to lowest temperature
C
         DO 5 IB=JNUM,1,-1
           IF(CTEMP(IA,IB) .EQ. 0.0D+00 .OR. 
     &         CTEMP(IA,IB) .GE. TISO .AND. JNOISO .EQ. 1)THEN
               GOTO 5
           ELSE
C              add to Schiel parameter for continuous cooling 
C              transformation. Set JY=1 if Scheil parameter > 1
               DT(IA)=((JT/COOL)/SHEATT(IA,IB))+DT(IA)
               IF(DT(IA) .GE. 1.0D+00)JY(IA)=1
           ENDIF
5        CONTINUE
C
C
C VOLB22 is now the actual fraction of bainite that forms
C VOLB & VOLB2 are the fractions allowed by T0 and T0'
C
      WRITE(*,6)VOLB22(IA),DT(IA),MST(IA),BST(IA)  
C
C*
      WRITE(12,26)VOLB22(IA),DT(IA),MST(IA),BST(IA)  
C*
C     if Schiel parameter < 1, set fraction of bainite to zero
      IF(JY(IA) .EQ. 0) GOTO 7
C
  4    CONTINUE

C
  7       IF(IA .EQ. 1 .AND. JY(IA) .EQ. 0) THEN
               AVOLB=0.0D+00
               GOTO 13
          ENDIF
C
C Find the actual volume fraction of bainite corresponding
C  to the case where Scheil parameter equal unity exactly
C
      SLOPE=(VOLB22(IA)-VOLB22(IA-1))/(DT(IA)-DT(IA-1))
      CONST=VOLB22(IA)-SLOPE*DT(IA)
      AVOLB=SLOPE+CONST
C
13    VAUST=1.0D+00-AVOLB
C
C Calculate martensite-start temperatures -
C  two methods, Khan & Bhadeshia and Koistenen & Marburger
      CALL MART(MST(IA),TCAMB,VAUST,VMART,VAUSTR)
      CALL MART2(MST(IA),TCAMB,VAUST,VMART2,VAUST2)
C
C Account for prior transformation to ferrite
C
          DUM2=1.0D+00-VPRIOR
          AVOLB=AVOLB*DUM2
          VAUST=VAUST*DUM2
          VMART=VMART*DUM2
          VMART2=VMART2*DUM2
          VAUSTR=VAUSTR*DUM2
          VAUST2=VAUST2*DUM2
C
C Accounting complete
C
C --------------------------------------------
C
C Calculations complete - only WRITE and FORMAT statments
C
       WRITE(*,11)CBAR,CC(2),CC(3),CC(4),CC(5),CC(6),CC(7),COOL
     &,TIMISO,TISO
C      WRITE(4,111)CBAR,CC(2),CC(3),CC(4),CC(5),CC(6),CC(7),COOL
       WRITE(4,111)CBAR,CC(2),CC(3),COOL    
     &,JNOISO,TIMISO,TISO,VPRIOR,CRICH,CALPHA,
     & AVOLB,VMART2,VAUST2,VMART,VAUSTR
      WRITE(*,10)VPRIOR,CRICH,CALPHA
     &,AVOLB,VMART,VAUSTR,VMART2,VAUST2,MST(IA)
c Added for British Steel purposes
      WRITE(2,11)CBAR,CC(2),CC(3),CC(4),CC(5),CC(6),CC(7),COOL
     &,TIMISO,TISO
      CALL HARD(VMART,VAUSTR,AVOLB,HV,CC(1))
      WRITE(2,101)VPRIOR,CRICH,CALPHA
     &,AVOLB,VMART,VAUSTR,MST(IA),HV
      WRITE(2,102)
C
C***  opens file to save some results
C*
      OPEN(UNIT=5, FILE="outbain3.dat")
C*
      WRITE(5,*) VPRIOR, AVOLB, VMART2, VAUST2
C*
      WRITE(5,*) "Ferrite, bainite, martensite, ret.austenite 
     &    (Koistiner & Marburger)"
C*
      CLOSE(5)
C*
      CLOSE(12)
C***
C
3     FORMAT(F8.0,D10.2)
6     FORMAT(//'VOLB22 = ', F5.2, '   Total Schiel sum = ',D10.2,
     &       '    Ms (C) = ',F5.0,'    Bs (C) = ',F5.0)
C*
26    FORMAT(F5.2,D10.2,F5.0,F5.0)
C*
10    FORMAT(' Volume fraction prior ferrite      =',5X,F7.2/
     &       ' C in austenite enriched by ferrite =',5X,F7.2,' wt.%'/
     &       ' C in ferrite                       =',5X,F7.3,' wt.%'/
     &       ' Volume fraction bainite            =',5X,F7.2/
     &       ' Volume fraction martensite         =',5X,F7.2,
     &' (Khan & Bhadeshia)'/
     &       ' Volume fraction retained austenite =',5X,F7.2,
     &' (Khan & Bhadeshia)'/
     &       ' Volume fraction martensite         =',5X,F7.2,
     &' (Koistinen & Marburger)'/
     &       ' Volume fraction retained austenite =',5X,F7.2,
     &' (Koistinen & Marburger)'//
     &       ' Ms of residual austenite,     C    =',5X,F7.2/)
101    FORMAT(' Volume fraction prior ferrite      =',5X,F7.2/
     &        ' C in austenite enriched by ferrite =',5X,F7.2,' wt.%'/
     &        ' C in ferrite                       =',5X,F7.3,' wt.%'/
     &       ' Volume fraction bainite            =',5X,F7.2/
     &       ' Volume fraction martensite         =',5X,F7.2,
     &' (Khan & Bhadeshia)'/
     &       ' Volume fraction retained austenite =',5X,F7.2,
     &' (Khan & Bhadeshia)'/
     &       ' Ms of residual austenite,     C    =',5X,F7.2/
     &'  HARDNESS                                 =',5X,F7.2/)
102   FORMAT('-----------------------------------------------'/)
11    FORMAT(///
     &' Carbon       = ',5X,F7.3,' wt.%'/
     &' Silicon      = ',5X,F7.3,' wt.%'/
     &' Manganese    = ',5X,F7.3,' wt.%'/
     &' Nickel       = ',5X,F7.3,' wt.%'/
     &' Molybdenum   = ',5X,F7.3,' wt.%'/  
     &' Chromium     = ',5X,F7.3,' wt.%'/
     &' Vanadium     = ',5X,F7.3,' wt.%'/
     &' Cooling rate = ',5X,F7.3,' K/s '//
     &' Isothermal Transformation Time s        = ',F10.0/
     &' Isothermal Transformation Temperature C = ',F10.0//)
111   FORMAT(/F8.3,2F7.2,2X,F7.3,' K/s  ','JTISO=',I2/
     &F9.2,' s',F7.0,' C',F7.2,F7.2,F7.3,6F7.2)
21    FORMAT(20X,'*** WARNING - No isothermal',
     &' bainite - Transformation temperature (',F6.0,
     &')'/20X,'              above the bainite-start ',
     & '(',F6.0,') of residual austenite ***')

C
C
C     GOTO 16
C
17       STOP
         END
C
C*****************************************************************
C
C AFEG AFEG LN ACTIVITY OF IRON IN GAMMA
      DOUBLE PRECISION FUNCTION AFEG(XEQ,AJ)
      DOUBLE PRECISION XEQ,AJ,DEQ,TEQ
      DEQ=DSQRT(1-2*(1+2*AJ)*XEQ+(1+8*AJ)*XEQ*XEQ)
      TEQ=5*DLOG((1-XEQ)/(1-2*XEQ))
      TEQ=TEQ+DLOG(((1-2*AJ+(4*AJ-1)*XEQ-DEQ)/(2*AJ*(2*XEQ-1)))**6)
      AFEG=TEQ
      RETURN
      END
C
C*****************************************************************
C
      SUBROUTINE ANALY(J8,J9,CONST,SLOPE,CORR,X,Y,JP)
      IMPLICIT REAL*8(A-H,K-Z),INTEGER(I,J)
      DOUBLE PRECISION X(100),Y(100)
      I1=J8
      J8=J8-J9
      AX=0.0D+00
      AY=0.0D+00
      AX2=0.0D+00
      AY2=0.0D+00
      AXY=0.0D+00
      DO 1 I=1,J8
      AX=AX+X(I)
      AY=AY+Y(I)
      AXY=AXY+X(I)*Y(I)
      AX2=AX2+X(I)*X(I)
      AY2=AY2+Y(I)*Y(I)
1     CONTINUE
      CONST=(AY*AX2-AX*AXY)/(J8*AX2-AX*AX)
      SLOPE=((J8*AXY-AX*AY)/(J8*AX2-AX*AX))
      CORR=(J8*AXY-AX*AY)/(DSQRT((J8*AX2-AX*AX)*
     &(J8*AY2-AY*AY)))
      IF(JP .EQ. 1)WRITE(*,2)CONST,SLOPE,CORR,JP
2     FORMAT('   INTERCEPT=',D12.4,'    SLOPE=',D10.3,
     &'    CORRELATION=',F8.4, I10)
      J8=I1
      RETURN
      END
C*******************************************************************************
      SUBROUTINE AXTO(H,S,H1,S1,XTO,T,W,W1,F,AJ,AJ1,STORE,J)
      IMPLICIT REAL*8(A-H,K-Z),INTEGER(I,J)
      J1=0
1     DFTO=FTO1(H,S,XTO,T,W,W1,H1,S1,F,AJ,AJ1)+STORE
      J1=J1+1
      IF (DABS(DFTO) .LE. 10.0D+00) GOTO 2
      G9=G91(XTO,T,W,W1,H1,S1,F,H,S,AJ,AJ1)
      IF (J1 .GE. 9) GOTO 2
      XTO=XTO-DFTO/G9
      IF (XTO .LE. 1.0D-12) GOTO 3
      GOTO 1
3     XTO=0.0000D+00
2     RETURN
      END
C
C*******************************************************************
      SUBROUTINE BAINTT(KTEMP,SHEART,C,AC,JT,JTHIGH,JTLOW,T10,T20,W,W1)
      IMPLICIT REAL*8 (A-H,K-Z), INTEGER (I,J)
      DOUBLE PRECISION C(8),AC(8),KTEMP(200),SHEART(200)
      AVER=2.5
      R=8.31432
      W1=48570.0
      J=0
C
         AC(1)=C(1)
         AC(2)=C(2)
         AC(3)=C(3)
         AC(4)=C(4)
         AC(5)=C(5)
         AC(6)=C(6)
         AC(7)=C(7)
      CALL OMEGA(AC,W,XBAR,T10,T20,0)
C     -------------------------------------
      DO 1 JDUM=JTLOW,JTHIGH,JT
      J=J+1
      KTEMP(J)=JDUM*1.0D+00
      CTEMP=KTEMP(J)-273.00D+00
      IF (KTEMP(J) .GE. 1000.0) THEN
          H1=105525.0D+00
          S1=45.34521D+00
      ELSE
          H1=111918.0D+00
          S1=51.44D+00
      ENDIF
      F=ENERGY(KTEMP(J),T10,T20)
      AJ=1.0D+00-DEXP(-W/(R*KTEMP(J)))
      A=CG(AC(1),KTEMP(J),W,R)
      AFE=AFEG(AC(1),AJ)
      CALL GMAAX(A1,A,W1,F,R,KTEMP(J),X,AFE,H1,S1)
      GMAX=R*KTEMP(J)*(A1-A)
      CALL TTTT(KTEMP(J),GMAX,SHEART(J),DIFFT)
1     CONTINUE
C     -------------------------------------
      RETURN
      END
C*******************************************************************************
      SUBROUTINE BSMS(T10,T20,W,W1,JTLOW,JTHIGH,
     & JT,BS,MS,X1,XTOSL,XTOCN,XTOSL2,XTOCN2,JP,JXTO)
      IMPLICIT REAL*8 (A-H,K-Z), INTEGER (I,J)
      DOUBLE PRECISION DCTEMP(200), FTO(200),XXTO(200),YXTO(200)
     &,VXTO(20),WXTO(20),ECTEMP(20)
      J1=0
      J2=0
      XTO=0.07D+00
      R=8.3143D+00
      H=38575.0D+00
      S=13.48D+00
      ITLOW=JTLOW
      ITHIGH=JTHIGH
      IT=JT
C
2     DO 1 J= ITLOW,ITHIGH,IT
      J1=J1+1
      KTEMP=J
      IF (KTEMP .GE. 1000.0) THEN
          H1=105525.0D+00
          S1=45.34521D+00
      ELSE
          H1=111918.0D+00
          S1=51.44D+00
      ENDIF
      F=ENERGY(KTEMP,T10,T20)
      AJ=1.0D+00-DEXP(-W/(R*KTEMP))
      AJ1=1.0D+00-DEXP(-W1/(R*KTEMP))
      FTO(J1)=FTO1(H,S,X1,KTEMP,W,W1,H1,S1,F,AJ,AJ1)
      DCTEMP(J1)=KTEMP-273.0D+00
1     CONTINUE
      CALL ANALY(J1,0,CONST,SLOPE,CORR,DCTEMP,FTO,JP)
      BS=(-400.0-CONST)/SLOPE
C
      XMS=X1
      IF(XMS .GT. 0.0594D+00)XMS=0.0594D+00
      MS=(-1086.9D+00 + 2941.8D+00*XMS - 9.6016D+05*XMS*XMS
     &+ 2.5382D+07*XMS*XMS*XMS - 1.8488D+08*XMS*XMS*XMS*XMS
     & - CONST)/SLOPE
C
c      IF(ITLOW .LT. 140+273)ITLOW=140+273
      ITHIGH=DINT(BS/20.0)*20+20+273
      IF((ITHIGH-ITLOW)/IT .LT. 5)ITHIGH=5*IT+ITLOW
      J2=J2+1
      J1=0
      IF(J2 .EQ. 1) GOTO 2
C
C
      I1=0
      BXTO=0.07
      CXTO=BXTO
      DO 4 I= 313, 900, 20
      I1=I1+1
      KTEMP=I
      IF (KTEMP .GE. 1000.0) THEN
          H1=105525.0D+00
          S1=45.34521D+00
      ELSE
          H1=111918.0D+00
          S1=51.44D+00
      ENDIF
      F=ENERGY(KTEMP,T10,T20)      
      AJ=1.0D+00-DEXP(-W/(R*KTEMP))
      AJ1=1.0D+00-DEXP(-W1/(R*KTEMP))
      DCTEMP(I1)=KTEMP-273.0
      CALL AXTO(H,S,H1,S1,BXTO,KTEMP,
     &W,W1,F,AJ,AJ1,0.0D+00,0)
      CALL AXTO(H,S,H1,S1,CXTO,KTEMP,
     &W,W1,F,AJ,AJ1,400.0D+00,0)
      XXTO(I1)=BXTO
      YXTO(I1)=CXTO
      IF(CXTO .EQ. 0.0D+00)GOTO 6
4     CONTINUE
C
6     JJJ=0
      II1=DINT(BS-80 +273)
      II2=DINT(BS+60 +273)
      DO 5 JJ=1,100
      IF(DCTEMP(JJ)+273.0 .GT. II2 .OR. DCTEMP(JJ)+273.0 .LT.
     & II1) THEN
      GOTO 5
      ELSE
      JJJ=JJJ+1
      VXTO(JJJ)=XXTO(JJ)
      WXTO(JJJ)=YXTO(JJ)
      ECTEMP(JJJ)=DCTEMP(JJ)
      ENDIF
5     CONTINUE
      CALL ANALY(JJJ,0,XTOCN,XTOSL,CORR,ECTEMP,VXTO,JP)
      CALL ANALY(JJJ,0,XTOCN2,XTOSL2,CORR,ECTEMP,WXTO,JP)
C
C
      RETURN
      END
C*****************************************************************************
      SUBROUTINE BOUND(A,B,C)
      DOUBLE PRECISION A,B,C,D
2      WRITE(*,1)B,C
1     FORMAT(12X,' Input value out of bounds'/
     &14X,' The limits are ',D12.4,' to ', D12.4///
     &14X,' Input new value, or 999 to continue with'/
     &14X,' original risky input                     ')
      CALL REED(D)
      IF(D .EQ. 999.0)GOTO 3
      A=D
      IF(A .LT. B .OR. A .GT. C)GOTO 2
3     RETURN
      END
C*********************************************************************
      SUBROUTINE BOUNDI(I,J1,J2)
      IMPLICIT INTEGER(I,J)
2      WRITE(*,1)J1,J2
1     FORMAT(12X,' Input value out of bounds'/
     &14X,' The limits are ',I5   ,' to ', I5  ///)
      CALL REEDI(I)
      IF(I .LT. J1 .OR. I .GT. J2)GOTO 2
      RETURN
      END
C*****************************************************************
C
C CG CG DOUBLE PRECISION FUNCTION GIVING LFG  LN(ACTIVITY) OF CARBON IN AUSTENITE
      DOUBLE PRECISION FUNCTION CG(X,T,W,R)
      DOUBLE PRECISION J,DG,DUMMY,T,R,W,X
      J=1-DEXP(-W/(R*T))
      DG=DSQRT(1-2*(1+2*J)*X+(1+8*J)*X*X)
      DUMMY=5*DLOG((1-2*X)/X)+6*W/(R*T)+((38575.0)-(
     &13.48)*T)/(R*T)
      CG=DUMMY+DLOG(((DG-1+3*X)/(DG+1-3*X))**6)
      RETURN
      END
C*****************************************************************
C     ENERGY ENERGY
      DOUBLE PRECISION FUNCTION ENERGY(T,T10,T20)
      DOUBLE PRECISION T,T10,T20,F,T7
      T7=T-100*T20
      IF (T7 .LT. 300) GOTO 1
      IF (T7 .LT. 700) GOTO 2
      IF (T7 .LT. 940) GOTO 3
      F=-8.88909+0.26557*(T7-1140)-1.04923D-3*((T7-1140)**2)
      F=F+2.70013D-6*((T7-1140)**3)-3.58434D-9*((T7-1140)**4)
      GOTO 4
1     F=1.38*T7-1499
      GOTO 4
2     F=1.65786*T7-1581
      GOTO 4
3     F=1.30089*T7-1331
4     ENERGY=(141.0*T10 + F)*4.187
      RETURN
      END
C
C*****************************************************************
C
      DOUBLE PRECISION FUNCTION FTO1
     &(H,S,X,T,W,W1,H1,S1,F,AJ,AJ1)
      DOUBLE PRECISION R,X,T,T60,ZEN1,ZEN2,ZENER,W,W1,H1,S1,F,
     &AJ,AJ1,D,D11,H,S
      D=DSQRT(1-2*(1+2*AJ)*X+(1+8*AJ)*X*X)
      D11=DSQRT(9-6*X*(2*AJ1+3)+(9+16*AJ1)*X*X)
      T60=T*(1-X)/(28080*X)
      IF (T60 .GT. 1.0) GOTO 18
      IF (T60 .GT. 0.25) GOTO 17
      ZEN1=0.2307+42.7974*T60-233.8631*T60*T60+645.4485*T60*T60*T60
     &-954.3995*(T60**4)+711.8095*(T60**5)-211.5136*(T60**6)
      ZEN2=-2.6702+45.6337*T60-225.3965*T60*T60+567.7112*(T60**3)
     &-771.6466*(T60**4)+538.1778*(T60**5)-151.3818*(T60**6)
      GOTO 16
17    ZEN2=1
      ZEN1=3.295
16    ZENER=(((ZEN2*X)**2)*(-50898.56)/(1-X))+ZEN1*T*X*0.6623741
      ZENER=ZENER*4.187
      GOTO 15
18    ZENER=0.0
15    R=8.31432
      FTO1=X*R*T*DLOG(X*X)+X*(H1-(H)-(S1-(S)
     &)*T+4*W1-6*W)-R*T*(1-X)*DLOG((1-X)**4)+5*R*T*(1-2*X)*DLOG(1-2*X)
     &-R*T*X*DLOG(((D-1+3*X)/(D+1-3*X))**6)-R*T*(1-X)*DLOG(((1-2*AJ+(4
     &*AJ-1)*X-D)/(2*AJ*(2*X-1)))**6)+3*R*T*X*DLOG(3-4*X)+R*T*
     &X*DLOG(((D11-3+5*X)/(D11+3-5*X))**4)+(1-X)*F+ZENER
      RETURN
      END
C
C*****************************************************************
C
      DOUBLE PRECISION FUNCTION G91(XTO,T,W,W1,H1,S1,F,
     &H,S,AJ,AJ1)
      DOUBLE PRECISION R,XTO,T,W,W1,H1,S1,F,AJ,AJ1,FD,FD1,DT5,
     &DT6,DZEN1,DZEN2,DZEN3,V1,V2,V3,V4,V5,V6,V7,V8,V9,G1,
     &DZEN6,DZEN7,DZEN8,H,S
      R=8.31432
      FD=DSQRT(1-2*(1+2*AJ)*XTO+(1+8*AJ)*XTO*XTO)
      FD1=DSQRT(9-6*XTO*(2*AJ1+3)+(9+16*AJ1)*XTO*XTO)
      DT5=28080*XTO/(1-XTO)
      DT6=T/DT5
      IF (DT6 .GT. 1.0) GOTO 48
      IF (DT6 .LT. 0.25) GOTO 47
      DZEN1=0.2307+42.7974*DT6-233.8631*(DT6**2)+645.4485*(DT6**3)
     &-954.3995*(DT6**4)+711.8095*(DT6**5)-211.5136*(DT6**6)
      DZEN2=-2.6702+45.6337*DT6-225.3965*(DT6**2)+567.7112*(DT6**3)
     &-771.6466*(DT6**4)+538.1778*(DT6**5)-151.3818*(DT6**6)
      GOTO 46
47    DZEN2=1
      DZEN1=3.295
46    DZEN3=(((DZEN2*XTO)**2)*(-50898.56)/(1-XTO))+DZEN1*T*XTO*
     &(0.6623741)
      DZEN3=DZEN3*4.187
      GOTO 45
48    DZEN3=0.0
45    V1=FD-1+3*XTO
      V2=FD+1-3*XTO
      V3=1-2*AJ+(4*AJ-1)*XTO-FD
      V4=2*AJ*(2*XTO-1)
      V5=FD1-3+5*XTO
      V6=FD1+3-5*XTO
      V7=(XTO-1-2*AJ+8*XTO*AJ)/FD
      V8=(9*XTO-9-6*AJ1+16*AJ1*XTO)/FD1
      V9=H1-(H)-(S1-(S))*T-6*W+4*W1
      G1=2+DLOG(XTO**2)+4+DLOG((1-XTO)**4)-10-DLOG((1-2*XTO)**10)
     &-DLOG((V1/V2)**6)-6*((XTO/V1)*(V7+3+V1*(3-V7)/V2))
     &+DLOG((V3/V4)**6)-6*((1-XTO)/V3)*(4*AJ*(1-(V3/V4))-V7)
     &+3*(DLOG(3-4*XTO)-4*XTO/(3-4*XTO))+DLOG((V5/V6)**4)+4*(XTO
     &/V5)*(V8+5+(V5/V6)*(5-V8))
      IF (DT6 .GT. 1.0) GOTO 90
      DZEN6=(-3.3948+13.6112*DT6-13.4376*(DT6**2))*T/(28080*
     &((1-XTO)**2))
      DZEN7=(-3.3118+15.7462*DT6-23.2449*(DT6**2))*T/(28080*
     &((1-XTO)**2))
      DZEN8=50898*((DZEN2*XTO)**2)/((1-XTO)**2)+(-50898*(2*
     &DZEN2*DZEN6*(XTO**2)+2*XTO*(DZEN2**2))/(1-XTO))+DZEN1
     &*T*0.6623741+DZEN7*T*XTO*0.6623741
      GOTO 91
90    DZEN8=0.0
91    G91=V9+R*T*G1+DZEN8-F
      RETURN
      END
C
C**********************************************************************
C
      SUBROUTINE GMAAX(A1,A,W1,F,R,T,X,AFE,H1,S1)
      IMPLICIT REAL*8(A-H,K-Z)
C CALCULATION OF THE OPTIMUM NUCLEUS C CONTENT AND ACTIVITY OF C IN
C FERRITE NUCLEUS
C
C Estimate a value for X
C February 1991
       X=6.3998D-07*T-3.027D-04
       IF(X .LT. 0.1D-07)X=1.0D-08
       IF(X .GT. 0.32D-03)X=0.32D-03
C Estimation complete
C
      AJ1=1.0D+00-DEXP(-W1/(R*T))
1     D1=DSQRT(9.0D+00-6.0D+00*X*(2.0D+00*AJ1+3.0D+00)
     &+(9.0D+00+16.0D+00*AJ1)*X*X)
      B1=((D1-3.0D+00+5.0D+00*X)/(D1+3.0D+00-5.0D+00*X))
      B2=((3.0D+00-4.0D+00*X)/X)
      B3=(H1 - S1*T + 4.0D+00*W1)/(R*T)

      A1=DLOG(B1*B1*B1*B1*B2*B2*B2)+B3 
C
      IF(X .GT. 1.0D-08)THEN
        A1FE=DLOG(1.0D+00-X)
      ELSE 
        A1FE=X
      ENDIF
C
      TEST=F+R*T*(A1FE-AFE) - R*T*(A1-A)
C
C Newton iteration
C
      IF (DABS(TEST) .GT. 10.0) THEN
        DA1=(3.0D+00*X/(3.0D+00-4.0D+00*X))*(
     &  (4.0D+00*X-3.0D+00)/(X*X)-4.0D+00/X)
        DA2=(0.5D+00/D1)*(-12.0D+00*AJ1-18.0D+00+
     &  18.0D+00*X+32.0D+00*AJ1*X)
        DA2=4.0D+00*(((DA2+5.0D+00)/(D1-3.0D+00+5.0D+00*X))
     &  -((DA2-5.0D+00)/(D1+3.0D+00-5.0D+00*X)))
        DA1=DA1+DA2
        DA1FE=1.0D+00/(X-1.0D+00)
        ERROR=TEST/(R*T*(DA1FE-DA1))
        IF (ERROR .GT. X) ERROR = 0.3D+00*X
        X=DABS(X-ERROR)
        GOTO 1
      ENDIF
C End of iteration
      RETURN
      END
C************************************************************************
C
      SUBROUTINE OMEGA(C,W,XBAR,T10,T20,J)
C SUBROUTINE TO CALCULATE THE CARBON CARBON INTERACTION ENERGY IN
C AUSTENITE, AS A FUNCTION OF ALLOY COMPOSITION.  BASED ON .MUCG18
C THE ANSWER IS IN JOULES PER MOL.   **7 OCTOBER 1981**
      DOUBLE PRECISION C(8),W,P(8),B1,B2,Y(8),T10,T20,B3,XBAR
      INTEGER U,J
      B3=0.0D+00
      IF(J .EQ. 1) GOTO 2
      C(8)=C(1)+C(2)+C(3)+C(4)+C(5)+C(6)+C(7)
      C(8)=100.0D+00-C(8)
      C(8)=C(8)/55.84D+00
      C(1)=C(1)/12.0115D+00
      C(2)=C(2)/28.09D+00
      C(3)=C(3)/54.94D+00
      C(4)=C(4)/58.71D+00
      C(5)=C(5)/95.94D+00
      C(6)=C(6)/52.0D+00
      C(7)=C(7)/50.94D+00
2     B1=C(1)+C(2)+C(3)+C(4)+C(5)+C(6)+C(7)+C(8)
      DO 107 U=2,7
      Y(U)=C(U)/C(8)
107   CONTINUE
      DO 106 U=1,8
      C(U)=C(U)/B1
106   CONTINUE
      XBAR=C(1)
      XBAR=DINT(10000.0D+00*XBAR)
      XBAR=XBAR/10000
      B2=0.0D+00
      T10=Y(2)*(-3)+Y(3)*2+Y(4)*12+Y(5)*(-9)+Y(6)*(-1)+Y(7)*(-12)
      T20=-3*Y(2)-37.5*Y(3)-6*Y(4)-26*Y(5)-19*Y(6)-44*Y(7)
      P(2)=2013.0341+763.8167*C(2)+45802.87*C(2)**2-280061.63*C(2)**3
     &+3.864D+06*C(2)**4-2.4233D+07*C(2)**5+6.9547D+07*C(2)**6
      P(3)=2012.067-1764.095*C(3)+6287.52*C(3)**2-21647.96*C(3)**3-
     &2.0119D+06*C(3)**4+3.1716D+07*C(3)**5-1.3885D+08*C(3)**6
      P(4)=2006.8017+2330.2424*C(4)-54915.32*C(4)**2+1.6216D+06*C(4)**3
     &-2.4968D+07*C(4)**4+1.8838D+08*C(4)**5-5.5531D+08*C(4)**6
      P(5)=2006.834-2997.314*C(5)-37906.61*C(5)**2+1.0328D+06*C(5)**3
     &-1.3306D+07*C(5)**4+8.411D+07*C(5)**5-2.0826D+08*C(5)**6
      P(6)=2012.367-9224.2655*C(6)+33657.8*C(6)**2-566827.83*C(6)**3
     &+8.5676D+06*C(6)**4-6.7482D+07*C(6)**5 +2.0837D+08*C(6)**6
      P(7)=2011.9996-6247.9118*C(7)+5411.7566*C(7)**2
     &+250118.1085*C(7)**3-4.1676D+06*C(7)**4
      DO 108 U=2,7
      B3=B3+P(U)*Y(U)
      B2=B2+Y(U)
108   CONTINUE
      IF (B2 .EQ. 0.0D+00) GOTO 455
      W=(B3/B2)*4.187
      GOTO 456
455   W=8054.0
456   CONTINUE
      RETURN
      END
C
C********************************************************************************
      SUBROUTINE RCOMP(C,ID,JYES)
      IMPLICIT REAL*8(A-H,K-Z),INTEGER(I,J)
      DOUBLE PRECISION C(8)

      IF(JYES .EQ. 1)GOTO 1
      IF(JYES .EQ. 2)GOTO 2
      IF(JYES .EQ. 3)GOTO 3
      IF(JYES .EQ. 4)GOTO 4
      IF(JYES .EQ. 5)GOTO 5
      IF(JYES .EQ. 6)GOTO 6
      IF(JYES .EQ. 7)GOTO 7

      WRITE(*,715)
715   FORMAT(////19X,' Identification Number ?')
      CALL REEDI(ID)
1     WRITE(*,707)
707   FORMAT(19X,' Carbon wt.% ?')
      CALL REED(C(1))
      IF(C(1) .LT. 0.1D-03 .OR. C(1) .GT. 2.0D+00)
     &CALL BOUND(C(1),0.1D-03,2.0D+00)
      IF(JYES .NE. 0) GOTO 8
2     WRITE(*,708)
      CALL REED(C(2))
      IF(C(2) .LT. 0.0D+00 .OR. C(2) .GT. 2.5D+00)
     &CALL BOUND(C(2),0.0D+00,2.5D+00)
708   FORMAT(19X,' Silicon wt.% ?')
      IF(JYES .NE. 0) GOTO 8
3     WRITE(*,709)
      CALL REED(C(3))
      IF(C(3) .LT. 0.0D+00 .OR. C(3) .GT. 3.5D+00)
     &CALL BOUND(C(3),0.0D+00,3.5D+00)
709   FORMAT(19X,' Manganese wt.% ?')
      IF(JYES .NE. 0) GOTO 8
4     WRITE(*,710)
      CALL REED(C(4))
      IF(C(4) .LT. 0.0D+00 .OR. C(4) .GT. 4.5D+00)
     &CALL BOUND(C(4),0.0D+00,4.5D+00)
710   FORMAT(19X,' Nickel wt.% ?')
711   FORMAT(19X,' Molybdenum wt.% ?')
      IF(JYES .NE. 0) GOTO 8
5     WRITE(*,711)
      CALL REED(C(5))
      IF(C(5) .LT. 0.0D+00 .OR. C(5) .GT. 1.5D+00)
     &CALL BOUND(C(5),0.0D+00,1.5D+00)
712   FORMAT(19X,' Chromium wt.% ?')
      IF(JYES .NE. 0) GOTO 8
6     WRITE(*,712)
      CALL REED(C(6))
      IF(C(6) .LT. 0.0D+00 .OR. C(6) .GT. 3.5D+00)
     &CALL BOUND(C(6),0.0D+00,3.5D+00)
713   FORMAT(19X,' Vanadium wt.% ?')
      IF(JYES .NE. 0) GOTO 8
7     WRITE(*,713)
      CALL REED(C(7))
      IF(C(7) .LT. 0.0D+00 .OR. C(7) .GT. 1.5D+00)
     &CALL BOUND(C(7),0.0D+00,1.5D+00)


8     RETURN
      END
C***********************************************************************
      SUBROUTINE REED(A)
      DOUBLE PRECISION A
996   READ(*,*,ERR=999)A
          GOTO 998
999       WRITE(*,997)
997       FORMAT(19X,' Incorrect Input. Try again'/)
          GOTO 996
998   RETURN
      END
C***********************************************************************
      SUBROUTINE REEDI(I)
      INTEGER I
996   READ(*,*,ERR=999)I
          GOTO 998
999       WRITE(*,997)
997       FORMAT(19X,' Incorrect Input. Try again'/)
          GOTO 996
998   RETURN
      END
C
C**********************************************************************
C
      SUBROUTINE TTTT(T,GMAX,SHEART,DIFFT)
      IMPLICIT REAL*8(A-H,K-Z),INTEGER(I,J)
      SHEART=DEXP((0.2432D+06/(8.31432*T))-0.135D+03+20.0*DLOG(T)
     &-5*DLOG(DABS(GMAX)))
      DIFFT=DEXP((0.6031D+06/(8.31432*T))-0.1905D+03+20.0*DLOG(T)
     &-4*DLOG(DABS(GMAX)))
      RETURN
      END
C
C*******************************************************************************
C XALPH XALPH DOUBLE PRECISION FUNCTION GIVING THE EQUILIBRIUM MOL.FRAC. CARBON IN ALPHA
C BASED ON MY PAPER ON FIRST ORDER QUASICHEMICAL THEORY, METSCI
      DOUBLE PRECISION FUNCTION XALPH(T)
      DOUBLE PRECISION T,CTEMP
      CTEMP=(T-273.0D+00)/900.0D+00
      XALPH=0.1528D-02-0.8816D-02*CTEMP+0.2450D-01*CTEMP*CTEMP
     &-0.2417D-01*CTEMP*CTEMP*CTEMP+
     &0.6966D-02*CTEMP*CTEMP*CTEMP*CTEMP
      RETURN
      END
C **************************************************************
C 
      SUBROUTINE MART(MS,TQ,VAUST,VMART,VAUSTR)
      IMPLICIT DOUBLE PRECISION (A-H,K-Z)
C guess a value for the volume fraction of martensite, VMART
      IF(MS-TQ .GT. 70.0D+00)THEN
       VMART=0.5D+00
      ELSE
       VMART=0.05D+00
      ENDIF
C
1     A=GMART(VMART,MS,TQ)
      B=GMDASH(VMART)
      IF(DABS(A) .GT. 0.01D+00)THEN
      VMART=DABS(VMART - A/B)
      IF(VMART .GE. 1.0D+00)VMART=0.99D+00  
      GOTO 1
      ENDIF
      VMART=VMART*VAUST
      VAUSTR=VAUST-VMART
      RETURN
      END
C **************************************************************
C 
      SUBROUTINE MART2(MS,TQ,VAUST,VMART,VAUSTR)
C     Koistinen and Marburger
      IMPLICIT DOUBLE PRECISION (A-H,K-Z)
C guess a value for the volume fraction of martensite, VMART
      VMART=1.0D+00-DEXP(-0.011*(MS-TQ))
      VMART=VMART*VAUST
      VAUSTR=VAUST-VMART
      RETURN
      END

C ***************************************************************
C  Function subroutine for estimating GMART  
      DOUBLE PRECISION FUNCTION GMART(VMART,MS,TQ)
      IMPLICIT REAL*8(A-H,K-Z), INTEGER(I,J)
      M=0.0029D+00
C  Khan and Bhadeshia, 1990
      GMART=(-(DLOG(1.0D+0-VMART))/VMART)-M*(MS-TQ)-1.0D+0
      RETURN
      END
C ***************************************************************
C  Function subroutine for estimating GDASH (derivative of GMART)
      DOUBLE PRECISION FUNCTION GMDASH(VMART)
      IMPLICIT DOUBLE PRECISION (A-H,L-Z)
      GMDASH=(1.0D+0/((1.0D+0-VMART)*VMART))
     &+((DLOG(1.0D+0-VMART))/(VMART*VMART))
      RETURN
      END
C ***************************************************************
      SUBROUTINE HARD(VMART,VAUST,VOLB,HV,XBAR)
      IMPLICIT REAL*8(A-H,K-Z), INTEGER(I,J)
C  XBAR IS AVERAGE CARBON, WT.%
      H0=356 + 1020*XBAR
      HMART=1020*VOLB*(XBAR-0.03)/(1.0-VOLB) + H0
      HB=480.0
      HAUST=240.0
      HV=VMART*HMART + VAUST*HAUST + VOLB*HB
      RETURN
      END
